package dev.fitko.fitconnect.core.http;

import com.fasterxml.jackson.databind.ObjectMapper;
import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.core.util.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.client.RestTemplate;

import java.net.InetSocketAddress;
import java.net.Proxy;
import java.util.List;

import static com.fasterxml.jackson.databind.DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES;

public class ProxyConfig {

    private static final Logger LOGGER = LoggerFactory.getLogger(ProxyConfig.class);

    private ApplicationConfig config;
    private String host;
    private int port;

    public ProxyConfig(ApplicationConfig config) {
        this.config = config;
        this.host = config.getHttpProxyHost();
        this.port = config.getHttpProxyPort();
    }

    ProxyConfig() {
        this(ApplicationConfig.builder().httpProxyHost(null).httpProxyPort(0).build());
    }

    public RestTemplate proxyRestTemplate() {
        return hasProxySet() ? getProxyRestTemplate() : getDefaultRestTemplate();
    }

    private RestTemplate getDefaultRestTemplate() {
        LOGGER.info("No proxy configured");
        final RestTemplate restTemplate = new RestTemplate();
        setupTemplate(restTemplate);
        return restTemplate;
    }

    private ProxyRestTemplate getProxyRestTemplate() {
        LOGGER.info("Using proxy {}", this);
        final var requestFactory = new SimpleClientHttpRequestFactory();
        final var proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(this.host, this.port));
        requestFactory.setProxy(proxy);

        final ProxyRestTemplate proxyRestTemplate = new ProxyRestTemplate(requestFactory);
        setupTemplate(proxyRestTemplate);
        return proxyRestTemplate;
    }

    private void setMappingConverter(final RestTemplate restTemplate) {
        final MappingJackson2HttpMessageConverter jacksonMessageConverter = new MappingJackson2HttpMessageConverter();
        jacksonMessageConverter.setObjectMapper(new ObjectMapper().configure(FAIL_ON_UNKNOWN_PROPERTIES, false));
        restTemplate.getMessageConverters().add(jacksonMessageConverter);
    }

    private void setLoggingInterceptor(final RestTemplate restTemplate) {
        final List<ClientHttpRequestInterceptor> interceptors = restTemplate.getInterceptors();
        interceptors.add(new ApiRequestInterceptor());
        interceptors.add(new UserAgentInterceptor(this.config));
        restTemplate.setInterceptors(interceptors);
    }

    private void setupTemplate(final RestTemplate restTemplate) {
        setLoggingInterceptor(restTemplate);
        setMappingConverter(restTemplate);
    }

    boolean hasProxySet() {
        return !Strings.isNullOrEmpty(this.host) && this.port > 0;
    }

    @Override
    public String toString() {
        return String.format("ProxyConfig {host='%s', port=%d}", this.host, this.port);
    }
}
